// Intrusive AVL

# ifndef INTRUSIVE_AVL_H_
# define INTRUSIVE_AVL_H_

# include <stddef.h>

// Node
struct node {
  struct node          *left, *right;
  int                   bal_factor;
};

// Comparing function type
typedef int (*cmp_fun_type)(struct node*, struct node*);

// Main wrapper
struct intrusive_avl {
  cmp_fun_type          cmp;
  struct node          *root;
};

// Getting data from node
# define get_data(type_, member_, node_ptr_)                    \
  ((type_*)((char*)(node_ptr_) - offsetof(type_, member_)))

// Init
static inline
void avl_init(struct intrusive_avl *tree, cmp_fun_type cmp) {
  tree->root = NULL;
  tree->cmp = cmp;
}

// Find
struct node* avl_find(struct intrusive_avl *tree, struct node *x);

// Insert
int avl_insert(struct intrusive_avl *tree,
               struct node *new_node, struct node **inserted_node);

// Delete
struct node* avl_delete(struct intrusive_avl *tree, struct node *x);

// Reste
void avl_reset(struct intrusive_avl *tree, void (*free_node)(struct node*));

# endif
