// Testing AVL

# include <assert.h>
# include <err.h>
# include <stddef.h>
# include <stdio.h>
# include <stdlib.h>

# include "intrusive_avl.h"

struct data {
  int           key;
  struct node   node;
};

int compare(struct node *a_, struct node *b_) {
  struct data          *a = get_data(struct data, node, a_);
  struct data          *b = get_data(struct data, node, b_);
  int                   ka = a->key, kb = b->key;
  if (ka == kb) return 0;
  if (ka < kb) return -1;
  return 1;
}

void free_node(struct node *a_) {
  struct data          *a = get_data(struct data, node, a_);
  free(a);
}

static
int balcheck(struct node *cur) {
  if (cur == NULL) return -1;
  if (cur->bal_factor < -1 || cur->bal_factor > 1)
    return -2;
  int h1 = balcheck(cur->left);
  if (h1 < -1) return -2;
  int h2 = balcheck(cur->right);
  if (h2 < -1) return -2;
  int h = 1 + (h1 > h2 ? h1 : h2);
  return (cur->bal_factor == h1 - h2) ? h : -2;
}

int balance_check(struct intrusive_avl *tree) {
  return balcheck(tree->root) > -2;
}

static
int fill_test_no_rand(size_t len) {
  struct intrusive_avl          tree_, *tree = &tree_;
  avl_init(tree, compare);
  int                           key = 0;
  for (; len > 0; len--, key++) {
    struct data                *data = calloc(1, sizeof (struct data));
    struct node                *target;
    data->key = key;
    assert(avl_insert(tree, &(data->node), &target));
    assert(&(data->node) == target);
    assert((target = avl_find(tree, &(data->node))) != NULL && get_data(struct data, node, target)->key == key);
  }
  assert(balance_check(tree));
  avl_reset(tree, free_node);
  return 1;
}

static
int fill_test_no_rand2(size_t len) {
  struct intrusive_avl          tree_, *tree = &tree_;
  avl_init(tree, compare);
  int                           key = 0;
  for (; len > 0; len--, key++) {
    struct data                *data = calloc(1, sizeof (struct data));
    struct node                *target;
    data->key = key;
    assert(avl_insert(tree, &(data->node), &target));
    assert(&(data->node) == target);
    assert((target = avl_find(tree, &(data->node))) != NULL && get_data(struct data, node, target)->key == key);
  }
  assert(balance_check(tree));
  while (key > 0) {
    key--;
    struct data search = {
      .key=key,
      .node= {
        .left=NULL,
        .right=NULL,
        .bal_factor=0
      }
    };
    struct node                *target;
    target = avl_delete(tree, &(search.node));
    assert(target != NULL && get_data(struct data, node, target)->key == key);
    assert(avl_find(tree, target) == NULL);
    free_node(target);
  }
  assert(tree->root == NULL);
  return 1;
}


int main() {
  assert(fill_test_no_rand(15));
  assert(fill_test_no_rand2(15));
  return 0;
}
